//
//  dumperViewController.m
//  PLOP examples
//
//  Created by Rainer Schaaf on 09.01.15.
//  Copyright (c) 2015 Rainer Schaaf. All rights reserved.
//

#import "dumperViewController.h"
#import "PLOP_ios/PLOP_objc.h"

@interface dumperViewController ()

@end

@implementation dumperViewController

@synthesize textView;

- (void)viewDidLoad {
    [super viewDidLoad];
    
    textView.text = @"";
    [self dumper];
}

- (void) dumper {
    
    // Find our documents directory
    NSString *infile = [[NSBundle mainBundle] pathForResource:@"PLOP-datasheet" ofType:@"pdf"];
    
    /* global option list */
    NSString *globaloptlist = @"";
    
    PLOP *plop = [[PLOP alloc] init];
    if (!plop) {
        [self displayError:@"PLOP out of memory error."];
        return;
    }
    
    @try
    {
        NSInteger count, pcosmode, plainmetadata;
        NSInteger objtype, i, doc;
        NSString *docoptlist = @"requiredmode=minimum";
        NSMutableString *buffer = [[NSMutableString alloc]init];
        
        [plop set_option:globaloptlist];
        
        if ((doc = [plop open_document:infile optlist:docoptlist]) == -1)
        {
            NSString *errorText = [NSString stringWithFormat:@"Error %ld in %@(): %@\n",
                                   (long)[plop get_errnum], [plop get_apiname], [plop get_errmsg]];
            [self displayError:errorText];
            return;
        }
        
        /* --------- general information (always available) */
        
        pcosmode = [plop pcos_get_number:doc path:@"pcosmode"];
        
        [buffer appendFormat:@"   File name: %@\n", [plop pcos_get_string:doc path:@"filename"]];
        
        [buffer appendFormat:@" PDF version: %@\n", [plop pcos_get_string:doc path:@"pdfversionstring"]];
        
        [buffer appendFormat:@"  Encryption: %@\n",
         [plop pcos_get_string:doc path:@"encrypt/description"]];
        
        [buffer appendFormat:@"   Master pw: %@\n", [plop pcos_get_number:doc path:@"encrypt/master"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"     User pw: %@\n",
         [plop pcos_get_number:doc path:@"encrypt/user"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"Text copying: %@\n",
         [plop pcos_get_number:doc path:@"encrypt/nocopy"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"  Linearized: %@\n",
         [plop pcos_get_number:doc path:@"linearized"] ? @"yes" : @"no"];
        
        if (pcosmode == 0)
        {
            [buffer appendString:@"Minimum mode: no more information available\n\n"];
            [plop close_document:doc optlist:@""];
            return;
        }
        
        /* --------- more details (requires at least user password) */
        
        [buffer appendFormat:@"PDF/X status: %@\n", [plop pcos_get_string:doc path:@"pdfx"]];
        
        [buffer appendFormat:@"PDF/A status: %@\n", [plop pcos_get_string:doc path:@"pdfa"]];
        
        [buffer appendFormat:@"    XFA data: %@\n",
         (int) [plop pcos_get_number:doc path:@"type:/Root/AcroForm/XFA"]
         != pcos_ot_null ? @"yes" : @"no"];
        
        [buffer appendFormat:@"  Tagged PDF: %@\n\n",
         [plop pcos_get_number:doc path:@"tagged"] ? @"yes" : @"no"];
        
        [buffer appendFormat:@"No. of pages: %d\n",
         (int) [plop pcos_get_number:doc path: @"length:pages"]];
        
        [buffer appendFormat:@" Page 1 size: width=%g, height=%g\n",
         [plop pcos_get_number:doc path:[NSString stringWithFormat:@"pages[%d]/width", 0]],
         [plop pcos_get_number:doc path:[NSString stringWithFormat:@"pages[%d]/height", 0]]];
        
        count = (NSInteger) [plop pcos_get_number:doc path:@"length:fonts"];
        [buffer appendFormat:@"No. of fonts: %ld\n", (long)count];
        
        for (i=0; i < count; i++)
        {
            if ([plop pcos_get_number:doc path:[NSString stringWithFormat:@ "fonts[%ld]/embedded", (long)i]])
                [buffer appendString:@"embedded "];
            else
                [buffer appendString:@"unembedded "];
            
            [buffer appendFormat:@"%@ font ",
             [plop pcos_get_string:doc path:[NSString stringWithFormat:@"fonts[%ld]/type", (long)i]]];
            [buffer appendFormat:@"%@\n",
             [plop pcos_get_string:doc path:[NSString stringWithFormat:@"fonts[%ld]/name", (long)i]]];
        }
        
        [buffer appendString:@"\n"];
        
        plainmetadata = (NSInteger) [plop pcos_get_number:doc path:@"encrypt/plainmetadata"];
        
        if (pcosmode == 1 && !plainmetadata && ((NSInteger) [plop pcos_get_number:doc path:@"encrypt/nocopy"]))
        {
            [buffer appendString:@"Restricted mode: no more information available\n\n"];
            [plop close_document:doc optlist:@""];
            return;
        }
        
        /* --------- document info keys and XMP metadata (requires master pw
         * or plaintext metadata)
         */
        
        count = (NSInteger) [plop pcos_get_number:doc path:@"length:/Info"];
        
        for (i=0; i < count; i++)
        {
            objtype = (NSInteger) [plop pcos_get_number:doc path:[NSString stringWithFormat:@"type:/Info[%ld]", (long)i]];
            
            [buffer appendFormat:@"%@: ", [plop pcos_get_string:doc path:[NSString stringWithFormat: @"/Info[%ld].key", (long)i]]];
            
            /* Info entries can be stored as string or name objects */
            if (objtype == pcos_ot_string || objtype == pcos_ot_name)
            {
                [buffer appendFormat:@"'%@'\n",
                 [plop pcos_get_string:doc path:[NSString stringWithFormat: @"/Info[%ld]", (long)i]]];
            }
            else
            {
                [buffer appendFormat:@"(%@ object)\n",
                 [plop pcos_get_string:doc path:[NSString stringWithFormat: @"type:/Info[%ld]", (long)i]]];
            }
        }
        
        [buffer appendFormat:@"\nXMP meta data: "];
        
        objtype = (NSInteger) [plop pcos_get_number:doc path:@"type:/Root/Metadata"];
        if (objtype == pcos_ot_stream)
        {
            NSData *contents;
            
            contents = [plop pcos_get_stream:doc optlist:@"" path:@"/Root/Metadata"];
            [buffer appendFormat:@"%lu bytes ", (unsigned long)contents.length];
            
            
            // todo convert contents to unicode
            //   (void) PLOP_utf8_to_utf16(plop, contents, "utf16", &len);
            NSString *unicode = [[NSString alloc] initWithData:contents encoding:NSUTF8StringEncoding];
            [buffer appendFormat:@"(%lu Unicode characters)\n\n", (unsigned long)unicode.length/2];
        }
        else
        {
            [buffer appendString:@"not present\n\n"];
        }
        
        [plop close_document:doc optlist:@""];
        
        textView.text = buffer;
    }
    
    @catch (PLOPException *ex) {
        NSString *errorText = [NSString stringWithFormat:@"Error %ld in %@(): %@\n",
                     (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]];
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"PLOP crashed" message:errorText
                                                           delegate:self cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
        [alertView show];
    }
    @catch (NSException *ex) {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:[ex name] message:[ex reason]
                                                           delegate:self cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
        [alertView show];
    }
    @finally {
        plop = nil;
    }
    return;
}

- (void) displayError: (NSString *) message {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"PLOP error" message:message delegate:nil cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
    [alertView show];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
