//
//  encryptViewController.m
//  PLOP examples
//
//  Created by Rainer Schaaf on 09.01.15.
//  Copyright (c) 2015 Rainer Schaaf. All rights reserved.
//

#import "encryptViewController.h"
#import "PLOP_ios/PLOP_objc.h"

@interface encryptViewController ()

@end

@implementation encryptViewController

@synthesize webView;

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // encrypt the document
    [self encrypt];
    
    // display the encrypted document
    NSArray *dirPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                            NSUserDomainMask, YES);
    NSString *docsDir = dirPaths[0];
    NSURL *url = [NSURL fileURLWithPath:[docsDir stringByAppendingPathComponent:@"PLOP-datasheet-encryted.pdf"]];
    NSMutableURLRequest *urlRequest = [NSMutableURLRequest requestWithURL:url];
    
    [webView loadRequest:urlRequest];
}


- (void) encrypt {
    
    // input files (here located in the mainBundle)
    NSString *infile  = [[NSBundle mainBundle] pathForResource:@"PLOP-datasheet" ofType:@"pdf"];
    NSString *in_password = @"";
    
    // outputfile (lets write it to the DocumentDirectory)
    NSArray *dirPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                            NSUserDomainMask, YES);
    NSString *docsDir = dirPaths[0];
    
    //parameters for the output document
    NSString *out_filename = [docsDir stringByAppendingPathComponent:@"PLOP-datasheet-encryted.pdf"];
    NSString *out_master  = @"DEMO";
    NSString *out_user    = @"demo";
    NSString *permissions = @"";
    
    // This is where input files live. Adjust as necessary.
    NSString *searchpath = [[NSBundle mainBundle] resourcePath];
    
    PLOP *plop = [[PLOP alloc] init];
    if (!plop) {
        [self displayError:@"PLOP out of memory error."];
        return;
    }
    
    @try {
        NSInteger doc;
        NSString *optlist;
        
        [plop set_option:[NSString stringWithFormat:@"searchpath {%@} ", searchpath]];
        
        // open unprotected input file without any password
        optlist = [NSString stringWithFormat:@"password {%@}", in_password];
        if ((doc = [plop open_document:infile optlist:optlist]) == -1) {
            return [self displayError:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]]];
        }
        
        // create the output file with a password
        optlist = [NSString stringWithFormat:@"masterpassword={%@} userpassword={%@} "
                   "permissions={%@} input=%ld", out_master, out_user, permissions, (long)doc];
        if (([plop create_document:out_filename optlist:optlist]) == -1) {
            return [self displayError:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]]];
        }
        
        // close input and output files
        [plop close_document:doc optlist:@""];
    }
    @catch (PLOPException *ex) {
        NSString *errorText = [NSString stringWithFormat:@"Error %ld in %@(): %@\n", (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]];
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"PLOP crashed" message:errorText
                                                           delegate:self cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
        [alertView show];
    }
    @catch (NSException *ex) {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:[ex name] message:[ex reason]
                                                           delegate:self cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
        [alertView show];
    }
    @finally {
        plop = nil;
    }
    return;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) displayError: (NSString *) message {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"PLOP error" message:message delegate:nil cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
    [alertView show];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
